module IntBiMap (
  IntBiMap
, empty
, findElement
, findInt
, lookupInt
, insert
, size
, nextInt
) where

import Data.Map.Strict (Map, (!))
import Data.Sequence (Seq, (|>))
import qualified Data.Map.Strict as Map
import qualified Data.Sequence as Seq

data IntBiMap a = IntBiMap {
  elementSequence :: Seq a
, elementIndexMap :: Map a Int
}

instance Show a => Show (IntBiMap a) where
  show = show . elementSequence

empty :: IntBiMap a
empty = IntBiMap {elementSequence = Seq.empty,
                  elementIndexMap = Map.empty}

findElement :: IntBiMap a -> Int -> a
findElement m = Seq.index (elementSequence m)

findInt :: Ord a => IntBiMap a -> a -> Int
findInt m x = elementIndexMap m ! x

lookupInt :: Ord a => IntBiMap a -> a -> Maybe Int
lookupInt m x = Map.lookup x (elementIndexMap m)

insert :: Ord a => a -> IntBiMap a -> IntBiMap a
insert x (IntBiMap {elementSequence = eS, elementIndexMap = eIM}) =
  IntBiMap { elementSequence = eS |> x
           , elementIndexMap = Map.insert x (Seq.length eS) eIM }

size :: IntBiMap a -> Int
size = Seq.length . elementSequence

nextInt :: IntBiMap a -> Int
nextInt = size
